function [h,delta] = burnpark(N,om,D,W)
% [h,delta] = burnpark(N,om,D,W)
% Complex Chebyshev FIR filter design using
% Burnside & Parks method
%
% h     filter impulse response
% delta maximum approximation error
% N     filter length
% om    frequency grid (0 <= om <= pi)
% D     complex desired frequency response on the grid om
% W     positive weighting function on the grid om
%
% example: length 41 lowpass, band edges [.4 .5]*pi,
% weighting 1 in passband and 10 in stopband,
% desired passband group delay 12 samples
%
% om=pi*[linspace(O, .4,400) ,linspace( .5,1 ,500)];
% D=[exp(-j*om(1:400)*12),zeros(1,500)];
% W= [ones( 1,400) , 10*ones (1,500) ];
% [h,delta] = burnpark(41,om,D,W);
%
% Author: Mathias C. Lang, Vienna University of Technology, Nov. 97

tol = 1e-3;
om = om(:); D = D(:); W = W(:); L=length(om);

% set up initial problem
L_ini = 2*N;
length(om) ;
I_ini = (1:max(fix((L-1)/(L_ini-1)),1):L);
om_ini = om(I_ini); D_ini = D(I_ini); W_ini = W(I_ini);

AR = cos(om_ini*(0:N-1)); AI = sin(om_ini*(0:N-1)); w = 1./W_ini;
br = real(D_ini); bi = imag(D_ini);
A = [-w -AR; -w AR; -w AI; -w -AI];
b = [-br; br; -bi; bi]; c = [1;zeros(N,1)];
x0 = [max(abs([br;bi] ./[w;w]));zeros(N,1)]; % feasible starting solution

% start iteration
while 1,

    % solve lp-problem
%     [x,lam] = lp(c,A,b,[],[],x0);
    
    x = linprog( [1;zeros(N,1)],A,b );
    
    delta = x(1)
    h = x(2:N+1);
    x0 = x;
    
    % compute complex error function
    E = W.*(D-freqz(h,1,om));
    magE = abs(E); argE = angle(E);
    maxerr = max(magE);
    x0(1) = maxerr; % make xO feasible for next iteration
    
    % compute local maxima of error magnitude
    Imax = locmax(magE);
    ommax = om(Imax); argEmax = argE(Imax);
    magDmax = abs(D(Imax)); argDmax = angle(D(Imax));
    Wmax = W(Imax);
    
    % make a plot
    plot(om/pi,magE,ommax/pi,magE(Imax), 'or'); grid
    xlabel('\omega/\pi'); title('Magnitude of Complex Error');
    drawnow
    
    % check stopping criterion
    if (maxerr-delta)/delta < tol, break; end
    
    % build new constraints
    Anew = -[1./Wmax cos(ommax*(0:N-1)+argEmax(: ,ones(1,N)))];
    bnew = -magDmax.*cos(argDmax-argEmax);
    
    % update constraint set
    A = [A;Anew]; b = [b;bnew];
end