function [h,w] = ccirls(N,om,omedge,D,W,dc,crit,mode,maxit)
% CCIRLS: constrained FIR filter design in the complex domain
% by an iterative reweighted least squares method
% h = ccirls(N,om,omedge,D,W,dc,crit,mode,maxit)
%
% h         filter impulse response
% N         filter length
% om        frequency grid (0 <= om <= pi)
% omedge    band edges; must be contained in om!
% D         complex desired frequency response on the grid om
% W         positive weighting function on the grid om
% de        positive constraint function on the grid om;
%           for intervals without constraints specify negative de
% crit      string variable defining the design criterion:
%           'cheb' for constrained Chebyshev design
%           'ls' for constrained least squares design
% mode      '0' for generalized Lawson (no envelopes)
%           '1' for piecewise constant envelopes
%           '2' for piecewise linear envelopes
% maxit     maximum number of iterations
% 
% EXAMPLE (constrained least squares):
% Low-delay bandpass filter with passband and stopband constraints
% om=pi*[linspace(0, .34,750),linspace(.4, .6,500),linspace(.66,1,750)];
% omedge=pi*[0, .34, .4, .6, .66,lJ;
% D=[zeros(1,750),exp(-j*30*om(751:1250)),zeros(1,750)];
% W=[500*ones(1,750),ones(1,500),500*ones(1,750)];
% dc=[.001*ones(1,750),.01*ones(1,500), .001*ones(1,750)];
% h = ccirls(100,om,omedge,D,W,dc, 'ls' ,2,50);
%
% Author: Mathias C. Lang, Vienna University of Technology, Aug. 98

om=om(:); omedge=omedge(:); D=D(:); W=W(:); dc=dc(:); %L = length(om);
if strcmp(crit,'cheb')==1, cheb=1; else cheb=0; end

% initialize, define
tol = 1e-5;
h = zeros(N,1); evec = exp(-1i*om); IB = find(dc>0); IC = [];
if isempty(IB) && ~cheb, h = lslevin(N,om,D,W); return; end
allconstr = 1; if length(IB) < length(dc), allconstr = 0; end
alpha = 1+.5*(mode>0)*(1+~cheb);
if cheb,
    if ~isempty(IB), W(IB) = 0; end
    IC = find(W);
end

% initial multipliers
lam = ~W;
if cheb,
    mu = ~lam; mu = mu/sum(mu);
    w = mu.*(W.^2) + lam;
else
    w = W + lam;
end

% iterate
for i=1:maxit,
    
    % compute weighted l2 solution
    hO = h;
    h = lslevin(N,om,D,w);
    
    % compute magnitude of complex error
    E = abs(polyval(h(N:-1:1),evec) - D);
    
    % stop if nothing changes
    dh = norm(h-hO)/norm(h);
    if dh < tol, break; end
    
    % update weights
    if ~isempty(IC),
        update = (W(IC).*E(IC)).^alpha;
        update = update/(mu(IC)'*update);
        plot(om(IC)/pi,update), hold on
        if mode,
            update = envelope(update,om(IC),omedge,mode);
            plot(om(IC)/pi,update, 'r:')
        end
        mu(IC) = mu(IC).*update;
    end
    if ~isempty(IB),
        update = (E(IB)./dc(IB)).^alpha;
        % check for infeasibility
        if ~cheb && allconstr,
            Imax = locmax(update);
            if all(update(Imax) > 1),
                disp('There is no feasible solution. Relax the constraints. ');
                break;
            end
        end
        plot(om(IB)/pi,update), hold on
        if mode,
            update = envelope(update,om(IB),omedge,mode);
            plot(om(IB)/pi,update, 'r:')
        end
        if cheb,
            lam(IB)=lam(IB).*update;
        else
            lam(IB)=max(lam(IB).*update + W(IB).*(update-1),0);
        end
    end
    
    if cheb, w = mu.*(W.^2) + lam;
    else W = W + lam; end
    xlabel('\omega/\pi');
    title('normalized magnitude of complex error');
    drawnow, hold off
end