function h = exch_ce(N,om,D,W,dc)
% EXCH_CE: least squares design of FIR filters with arbitrary magnitude
% and phase responses with constraints on the magnitude of the complex
% error function
% h = exch_ce(N,om,D,W,dc)
%
% h     filter impulse response
% N     filter length
% om    frequency grid (0 <= om <= pi)
% D     complex desired frequency response on the grid om
% W     positive weighting function on the grid om
% de    positive constraint function on the grid om;
%       for intervals without constraints specify negative de
%
% EXAMPLE:
% low-delay bandpass filter with passband and stopband constraints
% om=pi*[linspace(0, .34,750),linspace(.4, .6,500),linspace(.66,1,750)];
% D=[zeros(1,750),exp(-j*30*om(751:1250)),zeros(1,750)];
% W=[500*ones(1,750),ones(1,500),500*ones(1,750)];
% dc=[.001*ones(1,750),.01*ones(1,500), .001*ones(1,750)];
% h = exch_ce(100,om,D,W,dc);
%
% Author: Mathias C. Lang, Vienna University of Technology, Aug. 98

om=om(:); D=D(:); W=W(:); dc=dc(:); Lom=length(om);
tol = 1e-3; se = sqrt(eps);

% set up objective function: min h'*toeplitz(t)*h - 2*c'*h
t=zeros(N,1); c=t;
for i = 1:N,
    t(i) = W.'*cos((i-1)*om);
    c(i) = W.'*(real(D) .*cos((i-1)*om) - imag(D) .*sin((i-1)*om));
end
t=t/Lom; c=c/Lom;

% solve unconstrained L2 problem
[L,d] = invtoep(t);
for i=1:N, L(i:N,i)=L(i:N,i)*sqrt(d(i));end % inv(toeplitz(t))=L*L'
h = L*(L'*c); h_uc = h;

% prepare for iteration
Aact= []; bact= [] ;
IB = find(dc>=0); if isempty(IB), return; end
allconstr = 1; if length(IB) < length(dc), allconstr = 0; end
dc = dc(IB); Dm = abs(D(IB)); Dp = angle(D(IB)); om = om(IB);
evec = exp(-1i*om);

% iterate
while 1,

    % compute error maxima
    E = polyval(h(N:-1:1),evec) - D(IB);
    Em = abs(E); Ep = angle(E);
    Em_dc = Em-dc;
    Imax = locmax(Em_dc);
    % find violating maxima
    Iviol = find(Em_dc(Imax)>0); Iviol = Imax(Iviol);
    if length(Iviol)==length(Imax) && allconstr,
        disp('There is no feasible solution. Relax the constraints. ');
        break;
    end
    
    % make a plot
    plot(om/pi,Em_dc,om(Iviol)/pi,Em_dc(Iviol),'rx');
    title('constraint violation');
    xlabel('\omega/\pi'); grid, drawnow
    
    % check stopping criterion
    if all(Em_dc(Iviol)<=max(dc(Iviol)*tol,se)), break; end
    
    % formulate new constraints
    omviol = om(Iviol); Epviol = Ep(Iviol);
    Dmviol = Dm(Iviol); Dpviol = Dp(Iviol);
    Anew = cos(omviol*(0:N-1) +Epviol( : ,ones(N,1)));
    bnew = Dmviol.*cos(Dpviol-Epviol) + dc(Iviol);
    
    % solve subproblem (dual formulation)
    A=[Aact;Anew]; b=[bact;bnew]; nb=length(b);
    AL = A*L; H = AL*AL'; f = b-A*h_uc;
    for i=1:nb, H(i,i) = H(i,i)+se; end
    l = nnqmin(H,f);
    h = h_uc-L*(AL'*l);
    
    % find active constraints
    act = find(l>se); Aact = A(act,:); bact = b(act);
end
