function h = exch_mp2(N,om,D,W,dm,dp)
% EXCH_MP2: constrained least squares design of FIR filters with
% arbitrary magnitude and phase responses
%
% h = exch_mp2(N,om,D,W,dm,dp)
%
% h     filter impulse response
% N     filter length
% om    frequency grid (0 <= om <= pi)
% D     complex desired frequency response on the grid om
% W     positive weighting function on the grid om
% dm    positive magnitude constraint function on the grid om;
%       for intervals without constraints specify negative dm
% dp    positive phase constraint function on the grid om;
%       for intervals without constraints specify negative dp
%
% EXAMPLE:
% Low-delay bandpass filter with passband and stopband constraints
% om=pi*[linspace(O, .34,750),linspace(.4, .6,500),linspace(.66,1,750)];
% D=[zeros(1,750),exp(-j*30*om(751:1250)),zeros(1,750)];
% W=[500*ones(1,750),ones(1,500),500*ones(1,750)];
% dm=[.001*ones(1,750),.01*ones(1,500), .001*ones(1,750)]; dp=dm;
% h = exch_mp2(100,om,D,W,dm,dp);
%
% Author: Mathias C. Lang, Vienna University of Technology, Aug. 98
om=om(:); D=D(:); W=W(:); dm=dm(:); dp=dp(:);
tol = 1e-3; Dm = abs(D); Dp = angle(D); Lom = length(om);
se = sqrt(eps);

% set up objective function: min h'*toeplitz(t)*h - 2*c'*h
t=zeros(N,1); c=t;
for i = 1:N,
    t(i) = W.'*cos((i-1)*om);
    c(i) = W.'*(real(D) .*cos((i-1)*om) - imag(D) .*sin((i-1)*om));
end
t=t/Lom; c=c/Lom;

% solve unconstrained L2 problem
[L,d] = invtoep(t);
for i=1:N, L(i:N,i)=L(i:N,i)*sqrt(d(i));end % inv(toeplitz(t))=L*L'
h = L*(L'*c); h_uc = h;

% prepare for iteration
Aact= []; bact= []; I_lm= []; I_act_lm= []; n_lm=0;
dp(find(dp<0 & dm>=0 & Dm))=pi/8;
IB = find(dm>=0 | (dp>=0 & Dm));
if isempty(IB), return; end
dm = dm(IB); dp = dp(IB); Dm = Dm(IB); Dp = Dp(IB); om = om(IB);
Im = find(dm>=0); Impass = find(dm>=0 & Dm); Ip = find(dp>=0 & Dm);
Implot = find(dm>0); Ipplot = find(dp>0 & Dm);
evec = exp(-1i*om);

% iterate
while 1,
    
    % compute constraint maxima
    H = polyval(h(N:-1:1),evec); Hm = abs(H); Hp = angle(H);
    c_um = Hm(Im) - Dm(Im) - dm(Im);
    c_lm = -Hm(Impass) + Dm(Impass) - dm(Impass);
    c_up = princval(Hp(Ip) - Dp(Ip) - dp(Ip));
    c_lp = princval(-Hp(Ip) + Dp(Ip) - dp(Ip));
    Imax_um = locmax(c_um); Imax_lm = locmax(c_lm);
    Imax_up = locmax(c_up); Imax_lp = locmax(c_lp);
    
    % find violating maxima
    Iv_um = find(c_um(Imax_um)>0); Iv_um = Imax_um(Iv_um);
    Iv_lm = find(c_lm(Imax_lm)>0); Iv_lm = Imax_lm(Iv_lm);
    Iv_up = find(c_up(Imax_up)>0); Iv_up = Imax_up(Iv_up);
    Iv_lp = find(c_lp(Imax_lp)>0); Iv_lp = Imax_lp(Iv_lp);
    
    % make a plot
    if ~isempty(Implot) && ~isempty(Ipplot),
        subplot(2,1,1),
        plot(om(Implot)/pi,(Hm(Implot)-Dm(Implot))./dm(Implot));
        title('normalized magnitude error');
        xlabel('\omega/\pi'); grid;
        subplot(2,1,2),
        plot(om(Ipplot)/pi,princval(Hp(Ipplot)-Dp(Ipplot))./dp(Ipplot));
        title('normalized phase error');
        xlabel('\omega/\pi'); grid;
        drawnow
    end
    
    % check stopping criterion
    if all( [c_um(Iv_um)<=max(dm(Im(Iv_um))*tol,se); ...
            c_lm(Iv_lm) <=max(dm(Impass(Iv_lm) )*tol, se) ; ...
            c_up(Iv_up) <=max(dp(Ip(Iv_up))*tol,se); ...
            c_lp(Iv_lp) <=max(dp(Ip(Iv_lp))*tol,se)]),
    break; end

    % formulate new constraints
    if ~isempty(Iv_um),
        I_um = Im(Iv_um);
        A_um_new = cos(om(I_um)*(0:N-1)+Hp(I_um,ones(N,1)));
        b_um_new = Dm(I_um) + dm(I_um);
    else A_um_new=[]; b_um_new=[]; end
    if ~isempty(Iv_lm) || ~isempty(I_act_lm),
        I_lm = [Impass(Iv_lm) ; I_act_lm]; n_lm = length(I_lm);
        A_lm_new = -cos(om(I_lm)*(0:N-1)+Hp(I_lm,ones(N,1)));
        b_lm_new = -Dm(I_lm) + dm(I_lm);
    else A_lm_new= []; b_lm_new= []; end
    if ~isempty(Iv_up),
        I_up = Ip(Iv_up);
        A_up_new = -sin(om(I_up)*(0:N-1)+Dp(I_up,ones(N,1)))- ...
            tan(dp(I_up,ones(1,N))).*cos(om(I_up)*(0:N-1)+Dp(I_up,ones(N,1)));
        b_up_new = zeros(size(I_up));
    else A_up_new=[]; b_up_new=[]; end
    if ~isempty(Iv_lp),
        I_lp = Ip(Iv_lp);
        A_lp_new = sin(om(I_lp)*(0:N-1)+Dp(I_lp,ones(N,1)))- ...
            tan(dp(I_lp,ones(1,N))).*cos(om(I_lp)*(0:N-1)+Dp(I_lp,ones(N,1)));
        b_lp_new = zeros(size(I_lp));
    else A_lp_new= []; b_lp_new= []; end
    
    % solve subproblem (dual formulation)
    A=[A_lm_new;Aact;A_um_new;A_up_new;A_lp_new] ;
    b=[b_lm_new;bact;b_um_new;b_up_new;b_lp_new]; nb=length(b);
    AL = A*L; H = AL*AL'; f = b-A*h_uc;
    for i=1:nb, H(i,i) = H(i,i)+sqrt(eps); end
    l = nnqmin(H,f);
    h = h_uc-L*(AL'*l);
    % check for infeasibility
    if max(A*h-b) > sqrt(eps),
        disp('There is no feasible solution. Relax the constraints. ');
        break;
    end

    % find active constraints
    act = find(l>sqrt(eps));
    I_act_lm = I_lm(act(find(act<=n_lm)));
    act = act(find(act>n_lm));
    Aact = A(act,:); bact = b(act);
end