function h = lqp_ce(N,om,D,W,dc,p,rsize,crit)
% LQP_CE: complex Chebyshev or least squares FIR filter design
% with constraints on the magnitude of the complex error function
% by linear/quadratic programming
%
% h = LQP_CE(N,om,D,W,dc,p,rsize,crit)
%
% h     filter impulse response
% N     filter length
% om    frequency grid (0 <= om <= pi)
% D     complex desired frequency response on the grid om
% W     positive weighting function on the grid om
% dc    positive constraint function on the grid om;
%       for intervals without constraints specify negative dec
% p     integer linearization parameter (p >= 2)
%       (a circle in the complex plane is approximated by a
%       regular polygon with 2p vertices)
% rsize for rsize=l, the polygon approximates the circle from
%       the outside, for rsize=2 it approximates the circle from
%       the inside
% crit  string variable defining the design criterion:
%       'cheb' for constrained Chebyshev design
%       'Is' for constrained least squares design
%
% example: Low-delay lowpass filter with passband constraints
% om=pi*[linspace(0, .2,50),linspace(.3,1,200)];
% D=[exp(-j*10*om(1:50)),zeros(1,200)]; W=[ones(1,50),100*ones(1,200)];
% dc=[.02*ones(1 ,50) ,-ones(1 ,200)];
% h = lqp_ce(31,om,D,W,dc,4,1, 'ls')
% Author: Mathias C. Lang, Vienna University of Technology, July 98

om=om(:); D=D(:); W=W(:); dc=dc(:); L = length(om);
Dmag = abs(D); Dph = angle(D);
if strcmp(crit,'cheb')==1, cheb=1; else cheb=0; end
IB0 = find(dc==0); LB0 = length(IB0);
IB1 = find(dc>0); LB1 = length(IB1);
AB0 = []; bB0 = []; AB1 = []; bB1 = [];
p2 = 2*p; alpha = pi*(0:p2-1)/p;
if cheb, % constrained Chebyshev
    Imin = find(dc<0); Lmin = length(Imin);
    if Lmin,
        Amin = zeros(p2*Lmin,N); bmin = zeros(p2*Lmin,1);
        for i=1:p2,
            I = (i-1)*Lmin+(1:Lmin);
            Amin(I,:) = W(Imin,ones(N,1)).*cos(alpha(i) - om(Imin)*(0:N-1));
            bmin(I) = W(Imin).*Dmag(Imin).*cos(alpha(i) + Dph(Imin));
        end
    else Amin=[]; bmin=[];
    end
else % constrained least squares
    t=zeros(N,1); c=t;
    for i = 1:N,
        t(i) = W.'*cos((i-1)*om);
        c(i) = W.'*(real(D).*cos((i-1)*om) - imag(D).*sin((i-1)*om));
    end
    t=t/L; c=c/L;
end

% formulate constraints
if LB0, % equality constraints
    AB0= [cos(om(IB0)*(0:N-1));-sin(om(IB0)*(0:N-1))];
    bB0= [real(D(IB0));imag(D(IB0))];
end

if LB1, % inequality constraints
    AB1 = zeros(p2*LB1,N); bB1 = zeros(p2*LB1,1);
    U = dc; if rsize==2, U = U*cos(pi/p2); end
    for i=1:p2,
        I = (i-1)*LB1+(1:LB1);
        AB1(I,:) = cos(alpha(i) - om(IB1)*(0:N-1));
        bB1(I) = U(IB1) + Dmag(IB1).*cos(alpha(i) + Dph(IB1));
    end
end
AB = [AB0;AB1]; bB = [bB0;bB1];

% solve optimization problem
if cheb,
    % linear program
    A = [zeros(2*LB0+p2*LB1 ,1) , AB; -ones(p2*Lmin, 1) ,Amin]; b=[bB; bmin];

%    h = lp( [1;zeros(N,1)],A,b,[],[],[], 2*LB0);        % original code for Matlab5

    Aeq=A(1:2*LB0, : ); beq=b(1:2*LB0 );             % using wrapper for 'glpk'
    A=A(2*LB0+1:end, : ); b=b(2*LB0+1:end );
    
	if isempty(Aeq), Aeq=[]; end
	if isempty(beq), beq=[]; end
    
    h = linprog( [1;zeros(N,1)],A,b,Aeq,beq );

    h=h(2:N+1);
else
    % quadratic program
%    h = qp(toeplitz(t),-c,AB,bB, [], [], [] ,2*LB0);

% AB, bB
% t,c

    Aeq=AB(1:2*LB0, : ); beq=bB(1:2*LB0 );          
    A=AB(2*LB0+1:end, : ); b=bB(2*LB0+1:end );

%  h = qp ([], toeplitz(t),-c,
%                     Aeq,beq,
%                     [],[],
%                     - inf (1, length(b)),A,b);

	if isempty(Aeq), Aeq=[]; end
	if isempty(beq), beq=[]; end

    h = quadprog(toeplitz(t),-c, A,b, Aeq,beq);   % using wrapper for 'qp'
end
