function h = lqp_mp(N,om,D,W,dm,dp,p,rsize,crit)
% LQP_MP: complex Chebyshev or least squares FIR filter design
% with magnitude and phase constraints
% by linear/quadratic programming
%
% h = LQP_MP(N,om,D,W,dm,dp,p,rsize,crit)
%
% h     filter impulse response
% N     filter length
% om    frequency grid (0 <= om <= pi)
% D     complex desired frequency response on the grid om
% W     positive weighting function on the grid om
% dm    positive magnitude constraint function on the grid om (linear,
%       not dB); for intervals without magnitude constraints specify
%       negative dm
% dp    positive phase constraint function on the grid om (in radians);
%       stopband values irrelevant; for (passband) intervals without
%       phase constraints specify negative dp
% p     integer linearization parameter (p >= 2)
%       (a circle in the complex plane is approximated by a
%       regular polygon with 2p vertices)
% rsize parameter specifying size of feasible region; for rsize=l,
%       the feasible region is approximated from the outside; for
%       rsize=2 it is approximated from the inside
% crit  string variable defining the design criterion:
%       'cheb' for constrained Chebyshev design
%       'ls' for constrained least squares design
%
%EXAMPLE:
%Chirp All-Pass (Steiglitz, ASSP-29, pp. 171-176, 1981), length 31,
%magnitude and phase errors must not exceed 0.002 (linear/rad):
%N=31; om=pi*(0:500)/500; D=exp(-j*((N-1)/2*om+4/pi*(om-pi/2).^2));
%W=ones(size(D)); dm=.002*W; dp=.002*W;
%h = lqp_mp(N,om,D,W,dm,dp,4,1, 'ls')
%Author: Mathias C. Lang, Vienna University of Technology, July 98

om=om(:); D=D(:); W=W(:); dm=dm(:); dp=dp(:); L = length(om);
Dmag = abs(D); Dph = angle(D);
dp((dp<0) & (dm>=0) & Dmag)=pi/4;
if strcmp(crit,'cheb')==1, cheb=1; else cheb=0; end
Is0 = find(dm==0 & ~Dmag); Ls0 = length(Is0);
Is1 = find(dm>0 & ~Dmag); Ls1 = length(Is1);
Im = find(dm>=0 & Dmag); Lm = length(Im);
Ip = find(dp>=0 & Dmag); Lp = length(Ip);
p2 = 2*p; alpha = pi*(0:p2-1)/p;
if cheb, % constrained Chebyshev
    Imin = find(dm<0); Lmin = length(Imin);
    if Lmin,
        Amin = zeros(p2*Lmin,N); bmin = zeros(p2*Lmin,1);
        for i=1:p2, 
            I= (i-1)*Lmin+(1:Lmin);
            Amin(I,:) = W(Imin,ones(N,1)).*cos(alpha(i) - om(Imin)*(0:N-1));
            bmin(I) = W(Imin).*Dmag(Imin).*cos(alpha(i) + Dph(Imin));
        end
    else Amin=[]; bmin=[];
    end
else % constrained least squares
    t=zeros(N,1); c=t;
    for i = 1:N,
        t(i) = W.'*cos((i-1)*om);
        c(i) = W.'*(real(D).*cos((i-1)*om) - imag(D).*sin((i-1)*om));
    end
    t=t/L; c=c/L;
end

% formulate constraints
if Lm, % passband magnitude constraints
    U = Dmag(Im) + dm(Im); if rsize==2, U = U.*cos(dp(Im)); end
    L = Dmag(Im) - dm(Im); if rsize==1, L = L.*cos(dp(Im)); end
    Am = cos(om(Im)*(0:N-1) + Dph(Im,ones(N,1)));
    Am = [Am; -Am]; bm = [U; -L] ;
else Am=[]; bm=[];
end

if Lp, % phase constraints
    Ap1 = -sin(om(Ip)*(0:N-1) + Dph(Ip,ones(N,1)));
    Ap2 = tan(dp(Ip,ones(N,1))).*cos(om(Ip)*(0:N-1) + Dph(Ip,ones(N,1)));
    Ap = [Ap1-Ap2;-Ap1-Ap2]; bp = zeros(2*Lp,1);
else Ap=[]; bp=[];
end

% stopband constraints
if Ls0,
    As0 = [cos(om(Is0)*(0:N-1));-sin(om(Is0)*(0:N-1))];
    bs0 = [real(D(Is0));imag(D(Is0))];
else As0= []; bs0= [] ;
end

if Ls1,
    As1 = zeros(p2*Ls1,N); bs1 = zeros(p2*Ls1,1);
    U = dm(Is1); if rsize==2, U = U*cos(pi/p2); end
    for i=1:p2,
        I = (i-1)*Ls1+(1:Ls1);
        As1(I,:) = cos(alpha(i) - om(Is1)*(0:N-1));
        bs1(I) = U;
    end
else As1= []; bs1= [] ;
end

% solve optimization problem
A = [As0;As1;Am;Ap]; b = [bs0;bs1;bm;bp];
if cheb,
    % linear program
    A = [zeros(2*(Lm+Lp+Ls0)+p2*Ls1,1),A;-ones(p2*Lmin,1),Amin];
    b =  [b;bmin];

%    h = lp( [1;zeros(N,1)],A,b,[],[],[], 2*Ls0);        % original code for Matlab5

    Aeq=A(1:2*Ls0, : ); beq=b(1:2*Ls0 );             % using wrapper for 'glpk'
    A=A(2*Ls0+1:end, : ); b=b(2*Ls0+1:end );
    
	if isempty(Aeq), Aeq=[]; end
	if isempty(beq), beq=[]; end
    
    h = linprog( [1;zeros(N,1)],A,b,Aeq,beq );

    h=h(2:N+1);
else
    % quadratic program
%    h = qp(toeplitz(t),-c,A,b,[],[],[],2*Ls0);

    Aeq=A(1:2*Ls0, : ); beq=b(1:2*Ls0 );             % using wrapper for 'qp'
    A=A(2*Ls0+1:end, : ); b=b(2*Ls0+1:end );
    
	if isempty(Aeq), Aeq=[]; end
	if isempty(beq), beq=[]; end 
    
    h = quadprog(toeplitz(t),-c, A,b, Aeq,beq);
end
