function h = mpcheb(N,om,D,Wm,Wp,pp,ps)
% h = mpcheb(N,om,D,Wm,Wp,pp,ps)
% Complex Chebyshev FIR Filter Design using linear programming
% with independent weighting of magnitude and phase errors
%
% h filter impulse response
% N filter length
% om frequency grid (0 <= om <= pi)
% D complex desired frequency response on the grid om
% Wm positive magnitude weighting function on the grid om
% Wp positive phase weighting function on the grid om
%(irrelevant in the stopbands)
% pp,ps integer linearization parameters for passbands and stopbands
%(pp,ps >= 2);
%for complex Chebyshev approximation the linearization error is
%sec(pi/(2*p))-1;
%for magnitude/phase approximation choose pp=2;
%
% example (Chen & Parks): length 31 lowpass, band edges [.12 .24J*pi,
% weighting 1 in passband and 10 in stopband, desired passband group
% delay 12 samples, magnitude more important than phase
%
% om=pi*[linspace(0, .12,50),linspace(.24,1,250)];
% D=[exp(-j*om(1:50)*12),zeros(1,250)];
% Wm=[ones(1,50) ,10*ones(1 ,250)]; Wp=[.1*ones(1 ,50) ,zeros(1 ,250)];
% h = mpcheb(31,om,D,Wm,Wp,2,6);
%
% Author: Mathias C. Lang, Vienna University of Technology, Nov. 97

om = om(:); D = D(:); Wm = Wm(:); Wp = Wp(:); L = length(om);
Dmag = abs(D); Dph = angle(D);
pass = find(Dmag > 0); stop = find(Dmag == 0);
Lp = length(pass); Ls = length(stop);
Wp(pass) = Wp(pass)./Dmag(pass);

A = zeros(pp*Lp + ps*Ls,N); b = zeros(pp*Lp + ps*Ls,1);
alpha_p = pi*(0:pp-1)/pp; alpha_s = pi*(0:ps-1)/ps;

for i=1:pp, % passbands
    I = (i-1)*Lp+(1:Lp);
    A(I,:) = Wm(pass,ones(N,1)).* ...
        cos (om(pass) * (0: N -1) + Dph(pass, ones (N ,1) ) ) *cos (alpha_p(i)) + ...
        Wp(pass,ones(N,1)).* ...
        sin(om(pass)*(0:N-1) + Dph(pass,ones(N,1)))*sin(alpha_p(i));
    b(I) = Wm(pass).*Dmag(pass)*cos(alpha_p(i));
end

if Ls > 0,
    for i=1:ps, % stopbands
        I = pp*Lp + (i-1)*Ls+(1:Ls);
        A(I,:) = Wm(stop,ones(N,1)).*cos(alpha_s(i) - om(stop)*(0:N-1));
    end
end

A = [-ones(2*(pp*Lp+ps*Ls),1),[A;-A]]; b = [b;-b];
x0 = [max(b)+1 ;zeros(N ,1)]; % feasible initial solution

%h= Ip([1;zeros(N,l)],A,b,[],[],x0);
      % original code for Matlab5

%[s1,s2]=size(A);                                 % native octave code
%ctype=[repmat( 'U', [1,s1] )] ;
%vartype= repmat( 'C', [1,s2]);
%lb=ones(1,s2)*(-Inf); 
%h = glpk([1;zeros(N,1)],A,b,lb,[],ctype,vartype);

h = linprog( [1;zeros(N,1)],A,b);         % using wrapper for 'glpk'

h = h(2:N+1);
